#***************************************************************#
#*********************bank vole case study *********************#
#***************************************************************#

# Manuscript: "How to account for behavioral states in step-selection analysis: a model comparison"
# by Pohle, J., Signer, J., Eccard, J.A., Dammhahn, M. and Schlägel, U.E.


##### 0) load packages, data and functions #####

### packages
library(raster)
library(dplyr)
library(survival)
library(fitdistrplus)
library(parallel)
library(numDeriv)
library(CircStats)
library(MASS)

# case control data can be downloaded from 
# https://www.dropbox.com/scl/fo/12noduo54yjjuw9x12svf/h?dl=0&rlkey=mitq02h5f0rw42dkispzhouiy

# set path to folder including data and code
folderpath<-getwd() # adjust accordingly

# load data with replicate and ID information
data_replicate<-readRDS(file.path(folderpath,"Info_replicates.rds"))

# other functions
source(file.path(folderpath,"functions.R"))

# filename
filename<-"Results_case_study.RData"


##### 1) analysis preparation #####

# model settings and objects

# model objects
iSSA<-list() # iSSA
HMMiSSA_all<-list() # HMM-iSSA with different starting values
HMM_HMMiSSA_all<-list() # HMM-iSSA approximation for HMM
TSiSSA1<-TSiSSA2<-list() # TS-iSSA
# settings for starting values
nIV<-50
beta_set1<-c(-2,-1,-0.5,0)
beta_set2<-c(0,0.5,1,2)
shape0<-rate0<-d.h<-matrix(NA,nIV,2)
beta0<-list()

# number of cores for parallel computing:
n_cores<-10


##### 2) loop across all individuals #####

ID_voles<-paste(data_replicate$replicate,data_replicate$id_vole1,sep=".")
for(i_vole in 1:length(ID_voles)){
  print(paste("Individual",ID_voles[i_vole]))
  ### load and prepare case-control data ###
  vole_data<-readRDS(file.path(folderpath,"Vole_case_control.rds"))
  # select data for focal individual
  vole_data<-vole_data[which(vole_data$id_vole==ID_voles[i_vole]),]
  # add log step length and negative step length
  vole_data$log_sl_hmmissa<-log(vole_data$sl_hmmissa)
  vole_data$neg_sl_hmmissa<-(-vole_data$sl_hmmissa)
  vole_data$log_sl_tsissa<-log(vole_data$sl_tsissa)
  vole_data$neg_sl_tsissa<-(-vole_data$sl_tsissa)
  ### prepare estimation ###
  # model objects
  iSSA[[i_vole]]<-list()
  HMMiSSA_all[[i_vole]]<-list()
  # HMMiSSA_mle[[i_vole]]<-list()
  # Viterbi_HMMiSSA[[i_vole]]<-list()
  HMM_HMMiSSA_all[[i_vole]]<-list()
  # HMM_HMMiSSA_mle[[i_vole]]<-list()
  TSiSSA1[[i_vole]]<-list()
  TSiSSA2[[i_vole]]<-list()
  # number of other individuals
  num_voles<-data_replicate$number_voles[i_vole]-1
  ### iSSA ###
  # covariates and formula
  names_covs<-c(paste("occ",2:(num_voles+1),"_hmmissa",sep=""),
                "log_sl_hmmissa", "neg_sl_hmmissa", "strata(step_id)")
  formula_iSSA<-as.formula(paste("used ~", paste(names_covs,collapse="+")))
  # estimation
  try(mod_cl<-survival::clogit(formula_iSSA, data=vole_data) %>% summary)
  if(exists("mod_cl")){
    iSSA[[i_vole]]<-mod_cl
    rm(mod_cl)
  }else{
    iSSA[[i_vole]]<-NA
  }
  ### HMM-iSSA ###
  # random starting values
  seed_iv<-2468+10*i_vole
  set.seed(seed_iv)
  for(iv_run in 1:nIV){
    beta0[[iv_run]]<-cbind(sample(beta_set1,num_voles),
                           sample(beta_set2,num_voles))
    shape0[iv_run,]<-data_replicate$shape_all[i_vole]*c(runif(1,0.2,0.9),runif(1,1.1,3))
    rate0[iv_run,]<-shape0[iv_run,]*c(runif(2,0.25,2.5))
    d.h[iv_run,]<-runif(2,0.75,0.95)
  }
  delta0<-c(0.5,0.5)
  # a priori estimates for gamma distribution
  correction<-c(data_replicate$shape_all[i_vole],data_replicate$rate_all[i_vole])
  # parallel estimation across 50 starting values
  cl <- makeCluster(n_cores, type = "FORK")
  try(mod_hmmissa<-parLapply(cl,1:nIV,hmmissa_iv_par,formula=formula_iSSA,
                           data=vole_data, N=2,
                           beta=beta0, shape=shape0, rate=rate0, d.h=d.h, 
                           delta=delta0 , correction=correction,
                           ID="burst_id"))
  if(exists("mod_hmmissa")){
    HMMiSSA_all[[i_vole]]<-mod_hmmissa
    rm(mod_hmmissa)
  }else{
    stopCluster(cl)
    stop("Stop: Problem in HMM-iSSA estimation")
  }
  ### HMM-HMM-iSSA ###
  # formula:
  names_covs<-c("log_sl_hmmissa", "neg_sl_hmmissa","strata(step_id)")
  formula_HMM<-as.formula(paste("used ~", paste(names_covs,collapse="+")))
  # parallel estimation
  #
  try(mod_HMM_hmmissa<-parLapply(cl,1:nIV,hmmissa_iv_par,formula=formula_HMM, data=vole_data, 
                                 N=2, beta=NULL, shape=shape0, rate=rate0, d.h=d.h, 
                                 delta=delta0, correction=correction,
                                 ID="burst_id"))
  if(exists("mod_HMM_hmmissa")){
    HMM_HMMiSSA_all[[i_vole]]<-mod_HMM_hmmissa
    rm(mod_HMM_hmmissa)
  }else{
    stopCluster(cl)
    stop("Stop: Problem in estimation of HMM HMM-iSSA approximation")
  }
  ### TS-iSSA ###
  # formula
  names_covs<-c(paste("occ",2:(num_voles+1),"_tsissa",sep=""),
                "log_sl_tsissa", "neg_sl_tsissa", "strata(step_id)")
  formula_TSiSSA<-as.formula(paste("used ~", paste(names_covs,collapse="+")))
  ### state 1
  idx_state<-which(vole_data$HMM_state==1)
  try(mod_cl<-survival::clogit(formula_TSiSSA, data=vole_data[idx_state,]) %>% summary)
  if(exists("mod_cl")){
    TSiSSA1[[i_vole]]<-mod_cl
    rm(mod_cl)
  }else{
    TSiSSA1[[i_vole]]<-NA
  }
  ### state 2
  idx_state<-which(vole_data$HMM_state==2)
  try(mod_cl<-survival::clogit(formula_TSiSSA, data=vole_data[idx_state,]) %>% summary)
  if(exists("mod_cl")){
    TSiSSA2[[i_vole]]<-mod_cl
    rm(mod_cl)
  }else{
    TSiSSA2[[i_vole]]<-NA
  }
  stopCluster(cl)
}

save(iSSA,HMMiSSA_all,
     HMM_HMMiSSA_all,TSiSSA1,TSiSSA2,
     file=file.path(folderpath,filename))
