#***************************************************************#
#****************** Results simulation study *******************#
#***************************************************************#

# Manuscript: "How to account for behavioral states in step-selection analysis: a model comparison"
# by Pohle, J., Signer, J., Eccard, J.A., Dammhahn, M. and Schlägel, U.E.


##### 0) load packages, data and functions #####

### packages
library(dplyr)
library(survival)
library(raster)
library(tidyr)
library(ggplot2)
library(kableExtra)

# set path to folder including data and code
folderpath<-getwd() # adjust accordingly


# lanscape used
landscape_range<-10 # or 50
# step lengths sampling for control locations
dist_sl_control<-"gamma" # or "unif"

# colors for plotting
alpha<-0.3
colors<-c("black","#0072B2",'orange',"darkgray")#"#999999")
tcolor<-c(rgb(col2rgb("#999999")[1],col2rgb("#999999")[2],col2rgb("#999999")[3],
              maxColorValue=255,alpha=alpha*255),
          rgb(col2rgb("#0072B2")[1],col2rgb("#0072B2")[2],col2rgb("#0072B2")[3],
              maxColorValue=255,alpha=alpha*255),
          rgb(col2rgb('orange')[1],col2rgb('orange')[2],col2rgb('orange')[3],
              maxColorValue=255,alpha=alpha*255))


##### 1) data frame with estimation results of all three scenarios #####

# data frame with true underlying values
data_pars<-data.frame(scenario=1:3,
              beta_1=NA,beta_2=NA,
              kappa_1=NA,kappa_2=NA,
              shape_1=NA,shape_2=NA,
              rate_1=NA,rate_2=NA)

# loop across scenarios
for(scenario in 1:3){
  # load simulation results
  load(file.path(folderpath,paste("Results_range",landscape_range,"_",dist_sl_control,
                                            "_sampling_scenario",scenario,".RData",sep="")))
  # create data frame
  df_results <-data.frame(id=1:nSim)
  
  # add iSSA results
  for(m in 1:length(num_control_est)){
    est<-mapply(function(mod,par_gamma){
      pars<-mod$coefficients[,1]+c(0,0,par_gamma)
      p_val<-mod$coefficients[1,5]
      nllk<-(-mod$loglik[2])
      return(c(pars,p_val,nllk))
    },mod=iSSA[[m]],par_gamma=asplit(param_gamma, MARGIN = 1)) %>%
      t()
    colnames(est)<-paste(c("iSSA_beta_0_","iSSA_kappa_0_","iSSA_shape_0_","iSSA_rate_0_",
                           "iSSA_pv_0_","iSSA_nllk_"),
                         num_control_est[m],"_",scenario,sep="")
    df_results<-cbind(df_results,est)
  }
  
  # add HMM-iSSA results
  for(m in 1:length(num_control_est)){
    est<-sapply(HMM.iSSA[[m]],function(mod){
      pars<-c(mod$beta,mod$kappa,mod$shape,mod$rate)
      p_val<-mod$p_beta
      nllk<-mod$nllk
      return(c(pars,p_val,nllk))
    },mod=) %>%
      t()
    colnames(est)<-paste(c("HMMiSSA_beta_1_","HMMiSSA_beta_2_",
                           "HMMiSSA_kappa_1_","HMMiSSA_kappa_2_",
                           "HMMiSSA_shape_1_","HMMiSSA_shape_2_",
                           "HMMiSSA_rate_1_","HMMiSSA_rate_2_",
                           "HMMiSSA_pv_1_","HMMiSSA_pv_2_",
                           "HMMiSSA_nllk_"),
                         num_control_est[m],"_",scenario,sep="")
    df_results<-cbind(df_results,est)
  }
  
  # add TS-iSSA results
  for(m in 1:length(num_control_est)){
    est<-mapply(function(mod1,mod2,mod_HMM){
      if(!is.na(mod1[1])){
        pars1<-c(mod1$coefficients[,1]+c(0,0,mod_HMM$shape[1],1/mod_HMM$scale[1]))
        p_val1<-mod1$coefficients[1,5]
        nllk1<-(-mod1$loglik[2])
      }else{
        pars1<-rep(NA,4)
        p_val1<-nllk1<-NA
      }
      if(!is.na(mod2[1])){
        pars2<-c(mod2$coefficients[,1]+c(0,0,mod_HMM$shape[2],1/mod_HMM$scale[2]))
        p_val2<-mod2$coefficients[1,5]
        nllk2<-(-mod2$loglik[2])
      }else{
        pars2<-rep(NA,4)
        p_val2<-nllk2<-NA
      }
      # assign estimates to state 1 if only one TS-iSSA was fitted
      if(is.na(mod1[1]) & !is.na(mod2[1])){
        pars1<-pars2
        p_val1<-p_val2
        nllk1<-nllk2
        pars2<-rep(NA,4)
        p_val2<-nllk2<-NA
      }
      return(c(pars1,pars2,p_val1,p_val2,nllk1,nllk2))
    },mod1=TS.iSSA_1[[m]],mod2=TS.iSSA_2[[m]],mod_HMM=HMM) %>%
      t()
    colnames(est)<-paste(c("TSiSSA_beta_1_","TSiSSA_kappa_1_","TSiSSA_shape_1_","TSiSSA_rate_1_",
                           "TSiSSA_beta_2_","TSiSSA_kappa_2_","TSiSSA_shape_2_","TSiSSA_rate_2_",
                           "TSiSSA_pv_1_","TSiSSA_pv_2_","TSiSSA_nllk_1_","TSiSSA_nllk_2_"),
                         num_control_est[m],"_",scenario,sep="")
    df_results<-cbind(df_results,est)
  }
  assign(paste("df_results_scenario",scenario,sep=""),df_results)
  rm(df_results)
  # underlying parameters
  data_pars[scenario,]<-c(scenario,parameter$beta,parameter$kappa,parameter$shape,parameter$rate)
}


##### 2) plot estimates #####

# long format needed
df_ggplot<-data.frame(cbind(df_results_scenario1,df_results_scenario2,df_results_scenario3)) %>% 
  dplyr::select(!starts_with(c("id","iSSA_pv_0_","iSSA_nllk_",
                               "HMMiSSA_pv_1_","HMMiSSA_pv_2_","HMMiSSA_nllk_",
                               "TSiSSA_pv_1_","TSiSSA_pv_2_","TSiSSA_nllk_1_","TSiSSA_nllk_2_"))) %>% 
  pivot_longer(
  cols = iSSA_beta_0_20_1:TSiSSA_rate_2_500_3,
  names_to = c("model", "par_type", "state","num_cont","scenario"),
  names_pattern = "(.*)_(.*)_(.*)_(.*)_(.*)",
  values_to = "estimates"
)

data_pars <- data_pars %>%
  pivot_longer(
    cols = beta_1:rate_2,
    names_to = c("par_type", "state"),
    names_pattern = "(.*)_(.*)",
    values_to = "values"
  ) %>% pivot_wider(names_from = state, 
                    values_from = values,
                    names_prefix = "state_")



# add columns for plotting
# x-axis and scenario labels in facet plot
df_ggplot <- df_ggplot %>% mutate(x=case_when(model=="iSSA" ~ 1,
                                              model=="TSiSSA" & state=="1" ~ 2,
                                              model=="TSiSSA" & state=="2" ~ 3,
                                              model=="HMMiSSA" & state=="1" ~ 4,
                                              model=="HMMiSSA" & state=="2" ~ 5,
                                TRUE ~ 0)) %>%
  mutate(scenario_label=case_when(scenario=="1"~"Scenario 1\n(inactive-active)",
                                  scenario=="2"~"Scenario 2\n(switching preferences)",
                                  scenario=="3"~"Scenario 3\n(HMM)",
                                 TRUE~NA)) %>% 
  mutate(state=case_when(state==1 ~ "1",
                         state==2 ~ "2",
                         state==0 ~ "--",
                         TRUE~NA)) %>%
  mutate(mod_state=paste(model,state,sep=""))


data_pars$scenario_label<-rep(c("Scenario 1\n(inactive-active)",
                              "Scenario 2\n(switching preferences)",
                              "Scenario 3\n(HMM)"), each=4)

# to allow for two colors in lines showing true values
data_pars <- data_pars %>% mutate(state_rep=state_1,
                                  h1=1.5,
                                  h2=3.5)


p_box<-ggplot(df_ggplot, aes(x=x, y=estimates, group = interaction(mod_state,num_cont),
                      fill=state,color=state)) +
  geom_boxplot(outlier.size = 0.8) +
  scale_colour_manual(values = c("black",colors[2:3])) +
  scale_fill_manual(values = c(tcolor)) +
  facet_grid(par_type ~ scenario_label, scales = "free", switch = "y") + 
  scale_x_continuous(breaks = c(1,2.5,4.5), labels = c("iSSA","TS-iSSA","HMM-iSSA")) +
  scale_y_continuous(position="right", labels = scales::number_format(accuracy = 0.01)) +
  geom_vline(data =data_pars,aes(xintercept=h1),lwd=0.4) + #,lty=2
  geom_vline(data =data_pars,aes(xintercept=h2),lwd=0.4) + #,lty=2
  theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
        panel.border = element_rect(colour = "black", fill=NA, linewidth=0.5),
        panel.background = element_blank(),
        legend.position='bottom') + #legend.title=element_blank()
  theme(axis.title = element_text(size=12), 
        axis.text = element_text(size=12), strip.text = element_text(size=14),
        legend.title = element_text(size=14),legend.text = element_text(size=11),
        legend.key.size = unit(2,"line")) + 
  geom_hline(data =data_pars, aes(yintercept = state_1),col="#0072B2",lwd=0.5)+
  geom_hline(data =data_pars, aes(yintercept = state_2),col="orange",lwd=0.5)+
  geom_hline(data =data_pars, aes(yintercept = state_rep),col="#0072B2",lwd=0.2)+
  labs(x="",y="")

p_box
ggsave(file.path(folderpath,paste("Sim_estimates_range",landscape_range,".pdf")), width=10, height=9)


##### 3) p-values #####

alpha<-0.05

tab_pv<-data.frame(cbind(df_results_scenario1,df_results_scenario2,df_results_scenario3)) %>% 
  dplyr::select(starts_with(c("iSSA_pv_0_","HMMiSSA_pv_1_","HMMiSSA_pv_2_",
                              "TSiSSA_pv_1_","TSiSSA_pv_2_"))) %>%
  pivot_longer(
    cols = iSSA_pv_0_20_1 :TSiSSA_pv_2_500_3,
    names_to = c("model", "par_type", "state","num_cont","scenario"),
    names_pattern = "(.*)_(.*)_(.*)_(.*)_(.*)",
    values_to = "p_v"
  ) %>%
  # mutate(mod_state=paste(model,state,sep=""),sig=as.numeric(p_v>alpha)) %>%
  mutate(state=ifelse(state=="0","1",state),sig=as.numeric(p_v<alpha)) %>%
  group_by(model,state,num_cont,scenario) %>%
  summarise(n=sum(sig)) %>% 
  pivot_wider(names_from = c(scenario,state),
              values_from = n,
              names_prefix = "scenario_") %>%
  mutate(model=factor(model,levels=c("iSSA","TSiSSA","HMMiSSA"))) %>%
  arrange(model,as.numeric(num_cont)) %>%
  relocate(model,num_cont,scenario1_b1=scenario_1_1,scenario1_b2=scenario_1_2,
           scenario2_b1=scenario_2_1,scenario2_b2=scenario_2_2,
           scenario3_b1=scenario_3_1,scenario3_b2=scenario_3_2)

tab_pv


##### 4) Viterbi missclassification #####

tab_mc<-matrix(NA,3,8)
colnames(tab_mc)<-c(paste(rep(paste("HMMiSSA",num_control_est,sep="_"),each=2),rep(c("mean","sd"),3),sep="_"),
                    paste("HMM",c("mean","sd"),sep="_"))
rownames(tab_mc)<-paste("scenario",1:3,sep="_")
for(scenario in 1:3){
  # load simulated data
  load(file.path(folderpath,
                 paste("Data_range",landscape_range,"_scenario",scenario,".RData",sep="")))
  # load simulation results
  load(file.path(folderpath,paste("Results_range",landscape_range,"_",dist_sl_control,
                                            "_sampling_scenario",scenario,".RData",sep="")))
  # HMM-iSSA
  for(i_nce in 1:length(num_control_est)){
    mc<-mapply(function(s_vit,data){
      return(sum(s_vit!=data$state[2:999])/998)
    },s_vit=Vit_HMM.iSSA[[i_nce]],data=sim_dat)
    tab_mc[scenario,i_nce*2-1] <- mc %>% mean %>% round(4)
    tab_mc[scenario,i_nce*2] <- mc %>% sd %>% round(4)
  }
  mc<-mapply(function(s_vit,data){
    return(sum(s_vit!=data$state[2:999])/998)
  },s_vit=Vit_HMM,data=sim_dat) 
  tab_mc[scenario,7] <- mc %>% mean %>% round(4)
  tab_mc[scenario,8] <- mc %>% sd %>% round(4)
}
tab_mc<-tab_mc*100
tab_mc
