##### Simulation study #####

# Manuscript: "How to account for behavioral states in step-selection analysis: a model comparison"
# by Pohle, J., Signer, J., Eccard, J.A., Dammhahn, M. and Schlägel, U.E.


##### 0) load packages, functions and landscape raster #####

# packages
library(tictoc)
library(parallel)
library(CircStats)
library(circular)
library(numDeriv)
library(raster)
library(survival)
library(MASS)
library(fitdistrplus)
library(foreach)
library(doParallel)

# set path to folder including data and code
folderpath<-getwd() # adjust accordingly

# functions
source(file.path("functions.R"))

# load landscape raster
# landscape rasters can be downloaded from 
# https://www.dropbox.com/scl/fo/12noduo54yjjuw9x12svf/h?dl=0&rlkey=mitq02h5f0rw42dkispzhouiy
landscape_range <- 10 #50
load(file.path(folderpath,paste("landscape",landscape_range,".RData",sep="")))


##### 1) simulation settings #####

##### general settings

# number simulation runs
nSim<-100
# number of locations
Ts<-1000
# number of points used for data simulation (to approximate density)
num_control_sim<-100000
# number of states
N<-2
# parameter list for hidden Markov step-selection model
parameter<-list()
# transition probability matrix of the underlying Markov chain
parameter$gamma<-matrix(0.1,2,2)
diag(parameter$gamma)<-0.9
# initial distribution of the underlying Markov chain
parameter$delta<-c(0.5,0.5)


##### scenario-specific settings

# scenario to run
scenario<-1

### scenario 1
if(scenario==1){
  # shape and rate parameters of gamma step length distributions
  parameter$shape=c(1.2,2.5)
  parameter$scale=c(0.8,3.5)
  parameter$rate=1/parameter$scale
  # mean and concentration parameter of von Mises turning angle distributions
  parameter$mu=c(0,0)
  parameter$kappa=c(0.3,1)
  # selection coefficients
  parameter$beta<-matrix(c(0,2),1,2)
  # HMM state order
  HMM_order <- "mean_sl"
  # HMM-iSSA state order
  hmmissa_order <- "mean_sl"
}

### scenario 2
if(scenario==2){
  # shape and rate parameters of gamma step length distributions
  parameter$shape=c(2.5,2.5)
  parameter$scale=c(3.5,3.5)
  parameter$rate=1/parameter$scale
  # mean and concentration parameter of von Mises turning angle distributions
  parameter$mu=c(0,0)
  parameter$kappa=c(1,1)
  # selection coefficients
  parameter$beta<-matrix(c(-2,2),1,2)
  # HMM state order
  HMM_order <- "gamma"
  # HMM-iSSA state order
  hmmissa_order <- "beta"
}

### scenario 3
if(scenario==3){
  # shape and rate parameters of gamma step length distributions
  parameter$shape=c(1.2,2.5)
  parameter$scale=c(0.8,3.5)
  parameter$rate=1/parameter$scale
  # mean and concentration parameter of von Mises turning angle distributions
  parameter$mu=c(0,0)
  parameter$kappa=c(0.3,1)
  # selection coefficients
  parameter$beta<-matrix(c(0,0),1,2)
  # HMM state order
  HMM_order <- "mean_sl"
  # HMM-iSSA state order
  hmmissa_order <- "mean_sl"
}


##### 2) estimation settings and objects #####

# number of control/available steps M
num_control_est<-c(20,100,500)

# number of cores
n_cores<-10

# model formulas
# iSSA, HMM-iSSA and TS-iSSA
formula_issa <- used ~ res_1 + cos_ta + log_sl + neg_sl + strata(stepID)
# HMM approximated by HMM-iSSA
formula_HMM <- used ~ cos_ta + log_sl + neg_sl + strata(stepID)

# model objects
iSSA<-list() # iSSA
HMM.iSSA<-list() # HMM-iSSA
Vit_HMM.iSSA<-list() # Viterbi sequence for HMM-iSSAs
HMM<-Vit_HMM<-TS.iSSA_1<-TS.iSSA_2<-list() # HMM and TS-iSSA
HMM_HMM.iSSA<-list() # approximate HMM estimation using case-control data and HMM-iSSA

# gamma distribution objects
# (gamma distribution fitted to observed steps to generate control locations)
param_gamma<-matrix(NA,nSim,2)

# time for parameter estimation
time_est<-array(NA,dim=c(nSim/n_cores,length(num_control_est),5),
                dimnames = list(1:(nSim/n_cores), # partition of simulation runs
                                num_control_est, # number of control locations
                                c("HMM", "iSSA", "HMM-iSSA","HMM_HMM-iSSA","TS-iSSA")))

# simulated data objects
# filename to save data
dataname <- paste("Data_range",landscape_range,"_scenario",scenario,".RData",sep="")
# list of simulated data sets
sim_dat<-NULL
# time to generate data
time_sim<-c()
# initialise data saving
save(sim_dat,time_sim,
     file=file.path(folderpath,dataname))


##### 3) Parameter estimation #####

# initalise clusters for parallel computation
# Note: type = "FORK" does not work for Windows!
cl <- makeCluster(n_cores, type = "FORK")
# for non-parallel computations, parLapply can be replaced by lapply in the loop below

# sampling procedure of control locations
dist_sl_control<-"gamma" # "unif" for uniform sampling of control step length

# filename
filename <- paste("Results_range",landscape_range,"_",dist_sl_control,
                  "_sampling_scenario",scenario,".RData",sep="")

# loop across simulation runs
# nSim splitted to avoid large memory space use (due to large case-control data sets)

for(part in 1:(nSim/n_cores)){
  ### i) movement data
  print(paste("Movement data generation part=",part,sep=""))
  ind.start<-(part-1)*10+1:n_cores # index required for seeds and save due to splitting of nSim
  seed<-6789+ind.start*123 # seed for data generation
  tic()
  try(simdat_run <- parLapply(cl=cl,1:n_cores,simdata_hmmissa_par,
                             parameter=parameter, Ts=Ts,
                             landscape=landscape,seed=seed,
                             n_control=num_control_sim,
                             center=c(0,0),ext_hr=50))
  time.h<-toc()
  # save time to generate data
  time_sim[part]<-as.numeric(time.h$toc-time.h$tic)
  # check for errors
  if(!exists("simdat_run")){
    stopCluster(cl)
    rm(cl)
    save.image(file.path("Results",filename))
    stop('error in data simulation (simdat_run)')
  }
  # fit gamma distribution to step length data
  try(param_g<-t(parSapply(cl=cl,simdat_run,fit.gamma_par,Ts=Ts)))
  # check for errors
  if(!exists("param_g")){
    stopCluster(cl)
    rm(cl)
    save.image(file.path("Results",filename))
    stop('error in param_g')
  }
  ### 2) HMM estimation for TS-iSSA
  # HMM estimation
  dat_HMM<-lapply(simdat_run, function(dat,Ts) return(dat[-c(1,Ts),]),Ts=Ts)
  tic()
  try(HMM.h<-parLapply(cl=cl,dat_HMM,mle_HMM,
                           shape0=parameter$shape,
                           scale0=parameter$scale,
                           kappa0=parameter$kappa,
                           gamma0=parameter$gamma))
  time.h<-toc()
  time_est[part,,1]<-as.numeric(time.h$toc-time.h$tic)
  if(!exists("HMM.h")){
    stopCluster(cl)
    rm(cl)
    save.image(file.path("Results",filename))
    stop('error in HMM estimation')
  }
  # add fitted HMMs to HMM list object
  HMM<-c(HMM,HMM.h)
  # Viterbi decoding
  try(Vit_HMM.h<-parLapply(cl=cl,1:n_cores,viterbi_HMM_par,
                         dat_HMM,HMM.h,N=2))
  if(!exists("Vit_HMM.h")){
    stopCluster(cl)
    rm(cl)
    save.image(file.path("Results",filename))
    stop('error in Viterbi (HMM)')
  }
  # add Viterbi sequences to Viterbi list object
  Vit_HMM<-c(Vit_HMM,Vit_HMM.h)
  # delete HMM-data
  rm(dat_HMM)
  ### 3) Parameter estimation step selection models
  # loop across num_control_est (M=20,100,500)
  for(i_nce in 1:3){ 
    # current number of control locations: 
    num_cont<-num_control_est[i_nce]
    # set seed for case-control data
    seed<-1234+ind.start*123+i_nce*12
    # initalise model lists to save results for each M
    if(part==1){
      iSSA[[i_nce]]<-list()
      HMM.iSSA[[i_nce]]<-list()
      TS.iSSA_1[[i_nce]]<-list()
      TS.iSSA_2[[i_nce]]<-list()
      HMM_HMM.iSSA[[i_nce]]<-list()
      Vit_HMM.iSSA[[i_nce]]<-list()
    }
    # create case-control data
    print(paste('Simulation part=',part,', M=',num_cont,sep=''))
    try(data_cc <- parLapply(cl=cl,1:n_cores,dat_case_cont_par,data=simdat_run,
                          landscape=landscape,n_control=num_cont,Ts=Ts,seed=seed,
                          dist_sl=dist_sl_control,para_sampling=data.frame(param_g)))
    # check for errors
    if(!exists("data_cc")){
      stopCluster(cl)
      rm(cl)
      save.image(file.path("Results",filename))
      stop('error in case-control data generation (iSSA)')
    }
    # estimation iSSA
    tic()
    try(iSSA.h<-parLapply(cl=cl,data_cc,issa_par,formula=formula_issa))
    time.h<-toc()
    time_est[part,i_nce,2]<-as.numeric(time.h$toc-time.h$tic)
    if(!exists("iSSA.h")){
      stopCluster(cl)
      rm(cl)
      save.image(file.path("Results",filename))
      stop('error in iSSA (iSSA.h)')
    }
    # add iSSA.h to corresponding iSSA list
    iSSA[[i_nce]]<-c(iSSA[[i_nce]],iSSA.h)
    # HMM-iSSA
    # starting values
    gamma0<-matrix(c(0.9,0.1,0.1,0.9),N,N)
    delta0<-c(0.5,0.5)
    shape0<-(parameter$shape)
    rate0<-(parameter$rate)
    kappa0<-parameter$kappa
    beta0<-parameter$beta
    tic()
    try(HMM.iSSA.h<-parLapply(cl=cl,1:n_cores,hmmissa_par,formula=formula_issa,
                            data=data_cc,N=2,
                            beta0=beta0,gamma0=gamma0,
                            delta0=delta0,shape0=shape0,rate0=rate0, kappa0=kappa0,
                            sl_gamma=TRUE, ta_vonMises=TRUE,
                            correction=cbind(param_g,0),ordering=hmmissa_order,
                            stationary=FALSE,stepmax=150))
    time.h<-toc()
    time_est[part,i_nce,3]<-as.numeric(time.h$toc-time.h$tic)
    if(!exists("HMM.iSSA.h")){
      stopCluster(cl)
      rm(cl)
      save.image(file.path("Results",filename))
      stop('error in HMM-iSSA')
    }
    # add HMM.iSSA.h to corresponding HMM.iSSA list
    HMM.iSSA[[i_nce]]<-c(HMM.iSSA[[i_nce]],HMM.iSSA.h)
    # HMM_HMM.iSSA
    tic()
    try(HMM_HMM.iSSA.h<-parLapply(cl=cl,1:n_cores,hmmissa_par,formula=formula_HMM,
                             data=data_cc,N=2,
                             beta0=NULL,gamma0=gamma0,
                             delta0=delta0,shape0=shape0,rate0=rate0, kappa0=kappa0,
                             sl_gamma=TRUE, ta_vonMises=TRUE,
                             correction=cbind(param_g,0),ordering=hmmissa_order,
                             stationary=FALSE,stepmax=150))
    time.h<-toc()
    time_est[part,i_nce,4]<-as.numeric(time.h$toc-time.h$tic)
    if(!exists("HMM_HMM.iSSA.h")){
      stopCluster(cl)
      rm(cl)
      save.image(file.path("Results",filename))
      stop('error in HMM approximation (HMM.iSSA model)')
    }
    # add HMM_HMM.iSSA.h to corresponding HMM_HMM.iSSA list
    HMM_HMM.iSSA[[i_nce]]<-c(HMM_HMM.iSSA[[i_nce]],HMM_HMM.iSSA.h)
    rm(HMM_HMM.iSSA.h)
    # delete case-control data
    rm(data_cc)
    # TS-iSSA
    # case-control-data for TS-iSSA
    try(data_cc <- parLapply(cl=cl,1:n_cores,dat_TS_case_cont_par,data=simdat_run,
                             s.vit=Vit_HMM.h,landscape=landscape,n_control=num_cont,
                             Ts=Ts,seed=seed,mod_HMM=HMM.h))
    # check for errors
    if(!exists("data_cc")){
      stopCluster(cl)
      rm(cl)
      save.image(file.path("Results",filename))
      stop('error in case-control data generation (TS-iSSA)')
    }
    # TS-iSSA 
    tic()
    # state 1
    try(TSiSSA.h1<-parLapply(cl=cl,data_cc,tsissa_par,formula=formula_issa,
                                state=1,nstate_thresh=10))
    # state 2
    try(TSiSSA.h2<-parLapply(cl=cl,data_cc,tsissa_par,formula=formula_issa,
                                state=2,nstate_thresh=10))
    time.h<-toc()
    time_est[part,i_nce,5]<-as.numeric(time.h$toc-time.h$tic)
    if(!exists("TSiSSA.h1") | !exists("TSiSSA.h2")){
      stopCluster(cl)
      rm(cl)
      save.image(file.path("Results",filename))
      stop('error in TS-iSSA')
    }
    # add TS.iSSAs to corresponding TS.iSSA lists
    TS.iSSA_1[[i_nce]]<-c(TS.iSSA_1[[i_nce]],TSiSSA.h1)
    TS.iSSA_2[[i_nce]]<-c(TS.iSSA_2[[i_nce]],TSiSSA.h2)
    # Viterbi decoding HMM-iSSA
    # seed for Viterbi decoding case-control data
    seed<-54321+ind.start*10
    try(Vit_HMM.iSSA.h<-parLapply(cl=cl,1:n_cores,vit_hmmissa_par,mod_hmmissa=HMM.iSSA.h,
                           data=simdat_run,
                           landscape=landscape,formula=formula_issa,Ts=Ts,
                           n_control=num_cont,dist_sl=dist_sl_control,
                           para_sampling=data.frame(param_g),
                           seed=seed,ID=NULL))
    if(!exists("Vit_HMM.iSSA.h")){
      stopCluster(cl)
      rm(cl)
      save.image(file.path("Results",filename))
      stop('error in HMM-iSSA Viterbi decoding')
    }
    # add Vit_HMM.iSSA.h to corresponding Vit_HMM.iSSA list
    Vit_HMM.iSSA.h<-lapply(Vit_HMM.iSSA.h,function(v_list) return(v_list[[1]]))
    Vit_HMM.iSSA[[i_nce]]<-c(Vit_HMM.iSSA[[i_nce]],Vit_HMM.iSSA.h)
    # delete auxiliary model objects
    rm(Vit_HMM.iSSA.h,HMM.iSSA.h,TSiSSA.h1,TSiSSA.h2,iSSA.h)
  }
  # delete auxiliary model objects
  rm(HMM.h,Vit_HMM.h)
  # save gamma parameters for control locations
  param_gamma[ind.start,]<-param_g
  rm(param_g)
  # save simulated movement tracks
  load(file.path(folderpath,dataname))
  sim_dat<-c(sim_dat,simdat_run)
  save(sim_dat,time_sim,
       file=file.path(folderpath,dataname))
  rm(simdat_run,sim_dat)
  # save file
  save.image(file.path(folderpath,filename))
}
stopCluster(cl)
rm(cl)
save.image(file.path(folderpath,filename))
