#***************************************************************#
#****** necessary functions for simulation and case study ******#
#***************************************************************#

# Manuscript: "How to account for behavioral states in step-selection analysis: a model comparison"
# by Pohle, J., Signer, J., Eccard, J.A., Dammhahn, M. and Schlägel, U.E.


##### 0) required packages and auxiliary functions #####

# library(CircStats)
# library(circular)
# library(numDeriv) # for hessian function
# library(raster)
# library(MASS) # ginv for generalised inverse of hessian

# function to compute turning angles from three locations
# function input:
# * x: vector including the first coordinates of three consecutive locations
# * y: vector including the second coordinates of three consecutive locations
# function output:
# * turning angle

ta_vec <- function(x,y){
  if((x[1]==x[2] & y[1]==y[2])|
     (x[2]==x[3] & y[2]==y[3])){
    return(NA)
  }
  ta <- atan2(y[3]-y[2],x[3]-x[2])-atan2(y[2]-y[1],x[2]-x[1])
  if(ta<=(-pi)){
    ta <- ta + 2*pi
  }
  if(ta>pi){
    ta <- ta - 2*pi
  }
  return(ta)
}

# function to compute turning angles for multiple end-point locations
# function input:
# * x: vector including coordinates of first location
# * y: vector including coordinates of second location
# * Z: matrix including coordinates of third, i.e. end-point, locations;
#     one location per row
# function output:
# * vector of turning angles

ta_mat <- function(x, y, Z){
  D_xy <- c(y[1] - x[1], y[2] - x[2])
  D_yZ <- matrix(c(Z[,1] - y[1], Z[,2] - y[2]), ncol = 2, byrow = FALSE)
  ta <- atan2(D_yZ[,2], D_yZ[,1]) - atan2(D_xy[2], D_xy[1])
  # ensure that ta in [-pi,pi]
  for(i_ta in 1:length(ta)){
    if(!is.na(ta[i_ta])){
      while(ta[i_ta]<=(-pi)){
        ta[i_ta] <- ta[i_ta] + 2*pi
      }
      while(ta[i_ta]>pi){
        ta[i_ta] <- ta[i_ta] - 2*pi
      }
    }
  }
  return(ta)
}


##### 1) Data generation from hidden Markov step-selection (for simulations) #####

# this function generates a movement track on a given landscape-raster
# based on a 2-state hidden Markov step-selection model
# for the movement kernel, gamma distributed step length and von-Mises distributed turning angles are assumed
# the integral is approximated using randomly sampled control points from a uniform distribution on circle in Cartesian coordinates

# function input
# * parameter: list of model parameters containing
#             - beta: matrix of selection coefficients with N columns and p rows,
#                 where N is the number of states and p is the number of (habitat) covariates;
#                 each column belongs to one state
#             - shape: vector of shape values for the step-length gamma distribution
#                 with length N
#             - rate: vector of rate values for the step-length gamma distribution
#                 with length N
#             - mu: vector of mean values for the turning-angle von Mises distribution
#                 with length N
#             - kappa: vector of mean values for the turning-angle von Mises distribution
#                 with length N
#             - delta: initial distribution of the underlying Markov chain
#                 with length N
#             - gamma: transition probability matrix of the underlying Markov chain
#                 with dimension NxN
# * Ts: number of locations to generate
# * landscape: raster object including the relevant habitat feature values, 
#              can be a RasterStack object if several landscape features are used 
# * seed: seed number for set.seed; if NULL, no seed is used
# * n_control: number of control points to approximate integral, large number required
# * center: coordinates of the raster centre to initialise the movement track
# * ext_hr: range around the centre to initialise the movement track;
#           first and second locations are randomly sampled around the center coordinates 
#           using a uniform distribution
# function output:
# * matrix including locations, state, step length, turning angle 
# notes:
# * first two locations are not based on hidden Markov step selection model and might be discarded

simdata_hmmissa <- function(parameter, Ts, landscape, seed=NULL, n_control, center, ext_hr){
  # transpose beta
  parameter$beta<-t(parameter$beta)
  # initialise location and step objects
  loc <- data.frame(x=rep(NA,Ts), y=rep(NA,Ts)) # locations
  sl <- ta <- state <- rep(NA,Ts) # step length, turning angle and state vector
  # maximum radii for control point uniform sampling 
  # * 99.99%-Quantile of the sl gamma distributions
  R_max<-qgamma(0.9999,shape=parameter$shape,
                rate=parameter$rate)
  # set seed
  if(!is.null(seed)){
    set.seed(seed)
  }
  # initial location and state
  loc[1,] <- c(sample((center[1]-ext_hr):(center[1]+ext_hr),1),
               sample((center[2]-ext_hr):(center[2]+ext_hr),1))
  state[1]<-sample(1:2,1,prob = parameter$delta)
  # second location and state
  loc[2,] <- c(sample((center[1]-ext_hr):(center[1]+ext_hr),1),
               sample((center[2]-ext_hr):(center[2]+ext_hr),1))
  state[2]<-sample(1:2,1,prob = parameter$gamma[state[1],])
  # loop across time
  for (ts in 3:Ts){
    # sample random control points uniformly Cartesian product circle
    sl_cont<-sqrt(runif(n_control,0,1))*R_max[state[ts-1]] # sl
    head_cont<-runif(n_control,-pi,pi) # heading
    # derive locations
    loc_cont<-cbind(loc[ts-1,1]+sl_cont*cos(head_cont),
                  loc[ts-1,2]+sl_cont*sin(head_cont)) # control locations
    ta_control <- ta_mat(as.numeric(loc[ts-2,]),as.numeric(loc[ts-1,]),loc_cont) # ta
    # extract landscape values
    cov_land<-raster::extract(landscape,loc_cont)
    # movement kernel
    kern<-dgamma(sl_cont,shape=parameter$shape[state[ts-1]],
                 scale=parameter$scale[state[ts-1]])*
      dvm(ta_control, mu=parameter$mu[state[ts-1]], 
          kappa=parameter$kappa[state[ts-1]])/
      sl_cont
    # selection function
    wfun <- exp(as.matrix(cov_land)%*%parameter$beta[state[ts-1],])
    # product of movement kernel and selection function
    prob_vec<-kern*wfun
    # check for boundary problems -> NAs might occur in wfun if landscape feature is missing
    indNa<-which(!is.na(prob_vec))
    if(length(indNa)<n_control){
      print(paste("locations at raster boundary in t=",ts,sep=""))
    }
    # sample a new step
    temp <- sample(indNa, 1, prob=prob_vec[indNa])
    loc[ts,1:2] <-loc_cont[temp,]
    sl[ts-1]<-sl_cont[temp]
    ta[ts-1]<-ta_control[temp]
    state[ts]<-sample(1:2,1,prob=parameter$gamma[state[ts-1],])
  }
  return(cbind(loc,sl,ta,state))
}


##### 2) Generation case-control data sets #####

# functions to create case-control data sets for TS-/HMM- and standard iSSA

##### 2a) Case-control data sets for (HMM-)iSSAs #####

# function to create case-control data sets for (HMM-)iSSAs
# turning angles are drawn uniformly from (-pi,pi)
# step lengths are drawn either from a uniform distribution or gamma distribution

# function input:
# * data: location data frame with coordinate columns x and y
# * landscape: landscape raster with covariate values; if NULL, no covariate values are extracted
# * n_control: number of control locations
# * Ts: length of the data
# * seed: seed value for set.seed; if NUll, no seed is used
# * dist_sl: distribution to draw random step length, "gamma" and "unif" are allowed
# * para_sampling: parameter list for sampling step length;
#   for dist_sl="gamma", shape and rate are required, for dist_sl="unif", Radius is required
# function output:
# * case-control data frame including columns stepID, x0 and y0 (current location),
#   x1 and y1 (next location), sl (step length), ta (turning angle), 
#   heading0 (heading to location (x0,y0)),
#   heading1 (heading to location (x1,y1)), used (case-control indicator), 
#   log_sl (logarithm of sl), neg_sl (negative sl value), cos_ta (cosine of ta)

dat_case_cont<-function(data,landscape,n_control,Ts,seed, dist_sl, para_sampling){
  # set seed
  if(!is.null(seed)){
    set.seed(seed)  
  }
  # initialise case-control data set
  n_cases<-n_control+1
  dat_cc<-data.frame(matrix(NA,(Ts-1)*n_cases,10))
  colnames(dat_cc)<-c('stepID','x0','y0','x1','y1',
                    'sl','ta','heading0','heading1','used')
  for(t in 2:(Ts-1)){
    # stepID
    dat_cc$stepID[(t-1)*n_cases+1:n_cases]<-t
    #step
    dat_cc$sl[(t-1)*n_cases+1]<-data$sl[t]
    if(dist_sl=="unif"){
      dat_cc$sl[(t-1)*n_cases+2:n_cases]<-runif(n_control,0,para_sampling$Radius)
    }else if(dist_sl=="gamma"){
      dat_cc$sl[(t-1)*n_cases+2:n_cases]<-rgamma(n_control,
                                                 shape=para_sampling$shape,
                                                 rate=para_sampling$rate)
    }else{
      stop('dist_sl must be unif or gamma')
    }
    #heading0
    dat_cc$heading0[(t-1)*n_cases+1:n_cases]<-atan2(data$y[t]-data$y[t-1],
                                                    data$x[t]-data$x[t-1])
    # turning angle
    dat_cc$ta[(t-1)*n_cases+1]<-ta_vec(data$x[(t-1):(t+1)],data$y[(t-1):(t+1)])
    dat_cc$ta[(t-1)*n_cases+2:n_cases]<-runif(n_control,-pi,pi)
    #heading1
    dat_cc$heading1[(t-1)*n_cases+1]<-atan2(data$y[t+1]-data$y[t],
                                            data$x[t+1]-data$x[t])
    dat_cc$heading1[(t-1)*n_cases+2:n_cases]<-dat_cc$heading0[(t-1)*n_cases+1]+
      dat_cc$ta[(t-1)*n_cases+2:n_cases]
    #coordinates (x0,y0)
    dat_cc$x0[(t-1)*n_cases+1:n_cases]<-data$x[t]
    dat_cc$y0[(t-1)*n_cases+1:n_cases]<-data$y[t]
    #coordinates (x1,y1)
    dat_cc$x1[(t-1)*n_cases+1]<-data$x[t+1]
    dat_cc$y1[(t-1)*n_cases+1]<-data$y[t+1]
    
    dat_cc$x1[(t-1)*n_cases+2:n_cases]<-data$x[t]+
      dat_cc$sl[(t-1)*n_cases+2:n_cases]*cos(dat_cc$heading1[(t-1)*n_cases+2:n_cases])
    dat_cc$y1[(t-1)*n_cases+2:n_cases]<-data$y[t]+
      dat_cc$sl[(t-1)*n_cases+2:n_cases]*sin(dat_cc$heading1[(t-1)*n_cases+2:n_cases])
    #used
    dat_cc$used[(t-1)*n_cases+1]<-1
    dat_cc$used[(t-1)*n_cases+2:n_cases]<-0
  }
  # discard first row
  dat_cc<-dat_cc[-c(1:n_cases),]
  #resource values
  if(!is.null(landscape)){
    dat_cc<-as.data.frame(cbind(dat_cc,raster::extract(landscape,dat_cc[,4:5])))
    colnames(dat_cc)[11:ncol(dat_cc)]<-paste("res_",1:(ncol(dat_cc)-10),sep="")
  }
  dat_cc$log_sl<-log(dat_cc$sl)
  dat_cc$neg_sl<-(-dat_cc$sl)
  dat_cc$cos_ta<-cos(dat_cc$ta)
  return(dat_cc)
}


##### 2b) Case-control data sets for TS-iSSAs #####

# function to create case-control data sets for TS-iSSAs
# turning angles are drawn uniformly from (-pi,pi]
# step lengths are drawn from state-dependent HMM gamma distributions 
# according to Viterbi sequence

# function input:
# * data: location data frame with coordinate columns x and y
# * s.vit: Viterbi sequence
# * landscape: landscape raster with covariate values; if NULL, no covariate values are extracted
# * n_control: number of control locations
# * Ts: length of the data
# * seed: seed value for set.seed; if NUll, no seed is used
# * mod_HMM: HMM model object 
#   including state-dependent gamma distribution parameter vectors shape and rate
# function output:
# * case-control data frame including columns stepID, state (Viterbi-decoded HMM state),
#   x0 and y0 (current location), x1 and y1 (next location), sl (step length), 
#   ta (turning angle), heading0 (heading to location (x0,y0)),
#   heading1 (heading to location (x1,y1)), used (case-control indicator), 
#   log_sl (logarithm of sl), neg_sl (negative sl value), cos_ta (cosine of ta)

dat_TS_case_cont<-function(data,s.vit,landscape,n_control,Ts,seed, mod_HMM){
  if(!is.null(seed)){
    set.seed(seed)
  }
  n_cases<-n_control+1
  dat_cc<-data.frame(matrix(NA,(Ts-1)*n_cases,11))
  colnames(dat_cc)<-c('stepID','state','x0','y0','x1','y1',
                    'sl','ta','heading0','heading1','used')
  # state responsible for move from (x0,y0) to (x1,y1), i.e. step and turn
  # heading0: heading until (x0,y0)
  # heading1: heading from (x0,y0) to (x1,y1)
  # fill Viterbi sequence with NAs to simplify loop across t 
  state_seq<-c(NA,s.vit,NA)
  # loop across t
  for(t in 2:(Ts-1)){
    # stepID
    dat_cc$stepID[(t-1)*n_cases+1:n_cases]<-t
    #state
    dat_cc$state[(t-1)*n_cases+1:n_cases]<-state_seq[t]
    state_run<-state_seq[t]
    #step
    dat_cc$sl[(t-1)*n_cases+1]<-data$sl[t]
    dat_cc$sl[(t-1)*n_cases+2:n_cases]<-rgamma(n_control,
                                               shape=mod_HMM$shape[state_run],
                                               scale=mod_HMM$scale[state_run])
    # heading0
    dat_cc$heading0[(t-1)*n_cases+1:n_cases]<-atan2(data$y[t]-data$y[t-1],
                                                  data$x[t]-data$x[t-1])
    # turning angle
    dat_cc$ta[(t-1)*n_cases+1]<-ta_vec(data$x[(t-1):(t+1)],data$y[(t-1):(t+1)])
    dat_cc$ta[(t-1)*n_cases+2:n_cases]<-runif(n_control,-pi,pi)
    # heading1
    dat_cc$heading1[(t-1)*n_cases+1]<-atan2(data$y[t+1]-data$y[t],
                                          data$x[t+1]-data$x[t])
    dat_cc$heading1[(t-1)*n_cases+2:n_cases]<-dat_cc$heading0[(t-1)*n_cases+1]+
      dat_cc$ta[(t-1)*n_cases+2:n_cases]
    #coordinates (x0,y0)
    dat_cc$x0[(t-1)*n_cases+1:n_cases]<-data$x[t]
    dat_cc$y0[(t-1)*n_cases+1:n_cases]<-data$y[t]
    #coordinates (x1,y1)
    dat_cc$x1[(t-1)*n_cases+1]<-data$x[t+1]
    dat_cc$y1[(t-1)*n_cases+1]<-data$y[t+1]
    
    dat_cc$x1[(t-1)*n_cases+2:n_cases]<-data$x[t]+
      dat_cc$sl[(t-1)*n_cases+2:n_cases]*cos(dat_cc$heading1[(t-1)*n_cases+2:n_cases])
    dat_cc$y1[(t-1)*n_cases+2:n_cases]<-data$y[t]+
      dat_cc$sl[(t-1)*n_cases+2:n_cases]*sin(dat_cc$heading1[(t-1)*n_cases+2:n_cases])
    #used
    dat_cc$used[(t-1)*n_cases+1]<-1
    dat_cc$used[(t-1)*n_cases+2:n_cases]<-0
  }
  # discard first row
  dat_cc<-dat_cc[-c(1:n_cases),]
  #resource values
  if(!is.null(landscape)){
    dat_cc<-as.data.frame(cbind(dat_cc,raster::extract(landscape,dat_cc[,5:6])))
    colnames(dat_cc)[12:ncol(dat_cc)]<-paste("res_",1:(ncol(dat_cc)-11),sep="")
  }
  dat_cc$log_sl<-log(dat_cc$sl)
  dat_cc$neg_sl<-(-dat_cc$sl)
  dat_cc$cos_ta<-cos(dat_cc$ta)
  return(dat_cc)
}


##### 3) HMM-iSSA estimation #####

# code to apply HMM-(i)SSA estimation

### Data requirements ###

# Data requires a certain structure:
# * must contain a column for stepID (name of that column arbitrary)
# * must contain a column indicating if row belongs to observed or control location (0/1-coding)
# * rows belonging to the same stepID must be directly below each other, 
#   with used step of a certain stepID being first
# * can contain a column for ID, e.g. to separate different bursts and/or individuals (name of column can be arbitrary)

### Code structure ###

# Parameters are estimated using the function "mle_hmmissa". It embeds the maximisation of the log-likelihood-function
# (i.e., the minimisation of the negative log-likelihood function nlogLike) using nlm. To avoid parameters outside of their parameter space 
# (e.g. probabilities greater 1 or lower zero in the transition probability matrix, short tpm), 
# the parameters are transformed before the maximisation using the function n2w. 
# After the maximisation, parameters are back-transformed to their natural form using the function w2n. 

# When using gamma-distributed step length and/or von-Mises distributed turning angles, 
# the code allows to constrain the overall shape, rate and kappa parameters to values greater than 0 
# using the arguments "sl_gamma=TRUE" and "ta_vonMises=TRUE", respectively. 
# Here, the function argument "correction" comes into play which is a vector of three elements and includes the values for shape, 
# rate and kappa that would be used in the standard iSSA to correct the parameter estimates 
# (e.g. a priori estimates for gamma distr., 1 for shape for uniform sampling) 

# The function "hessian_hmmissa" can be used to compute the hessian matrix of the negative log-likelihood function 
# at the maximum likelihood estimates using the hessian-function of the numDeriv-package. 
# The inverse of this hessian is used as an estimate for the variance-covariance matrix of the UNCONSTRAINED working parameter vector. 
# The square root of its diagonal give the standard error estimates and are used to compute p-values and confidence intervals for the selection coefficients.

# 7 functions for HMM-iSSA estimation:
# a) cpr_Nt: internal function; computes probabilities for Markov-switching conditional logistic regression for one step and $N$ states
# b) n2w_hmmissa: internal function; transformation from natural (constraint) parameters to working (unconstrained) parameter vector
# c) w2n_hmmissa: internal function; back-transformation from working (unconstrained) parameter vector to natural (constraint) parameters
# d) nlogLike_hmmissa: internal function; negative log-likelihood function, uses w2n_hmmissa and cpr_Nt
# e) mle_hmmissa: maximum likelihood estimation, uses n2w_hmmissa, w2n_hmmissa and nlogLike_hmmissa; maximisation based on nlm
# f) hessian_hmmissa: hessian at maximum likelihood estimates and hessian based SEs and CIs, uses n2w_hmmissa, w2n_hmmissa and nlogLike_hmmissa
# g) viterbi_hmmissa: State decoding using Viterbi algorithm


### 3a) cpr_Nt #####

# function to compute conditional logistic regression probabilities
# function input
# * X: design matrix corresponding to one stepID; first row belongs to observed step; dimension $M \times p$
# * beta_clr: coefficient matrix containing the selection and movement coefficients for each covariate and state; dimension $p \times N$
# function output:
# * clr probabilities for each state, respectively; vector of length $N$
# notes:
# * sapply/lapply can be used together with list of X matrices to calculate probabilities for each state and each time point in only one line of code (as used in function nlogLike)
# * internal function

cpr_Nt <- function(X,beta_clr){ 
  cpr<-exp(X%*%beta_clr) # exp of matrix product (Mxp %*% pxN)
  cpr<-cpr[1,]/apply(cpr,2,sum) # fraction: exp(X_used %*% beta_clr) in numerator, sum in denominator
  return(cpr)
}


##### 3b) n2w_hmmissa (internal function) #####

# transformation from natural (constraint) to working (unconstraint) parameter vector

# function input:
# * N: number of states for the underlying latent MC
# * beta: matrix of selection coefficients, no constraints applied; dimension $p\times N$ for sl_gamma=ta_vonMises=FALSE 
#         or $p* \times N$ for sl_gamma=TRUE and/or ta_vonMises=FALSE
# * gamma: tpm of underlying latent MC, all elements lie in (0,1) and rows sum to 1; dimension $N\times N$
# * delta: initial distribution for MC, NULL if stationarity is assumed, otherwise probability vector with elements in (0,1) which sum to 1; dimension $1\times N$
# * shape: shape parameter vector for gamma dist. in constraint (sl_gamma=TRUE) estimation, NULL otherwise; dimension $1\times N$
# * rate: rate parameter vector for gamma dist. in constraint (sl_gamma=TRUE) estimation, NULL otherwise; dimension $1\times N$
# * kappa: kappa parameter vector for von Mises dist. in constraint (ta_vonMises=TRUE) estimation, NULL otherwise; dimension $1\times N$
# * stationary: logical, TRUE if stationarity for underlying latent MC is assumed
# function output:
# * parameter vector of unconstrained parameters
# notes:
# * internal function

n2w_hmmissa<-function(N, beta, gamma, delta, shape=NULL, rate=NULL, kappa=NULL, 
              stationary=FALSE){
  # no transformation needed for beta
  tbeta<-beta
  # add log of kappa to tbeta matrix if kappa is not NULL 
  # (log transformation assures kappa>0 in estimation process)
  if(!is.null(kappa)){
    tkappa<-log(kappa) 
    tbeta<-rbind(tbeta,tkappa)
  }
  # add log of shape and rate to tbeta matrix if shape is not NULL 
  # (log transformation assures shape>0 and rate>0 in estimation process)
  if(!is.null(shape)){
    tshape<-log(shape)
    trate<-log(rate)
    tbeta<-rbind(tbeta,tshape,trate)
  }
  # mlogit transformation for tpm to assure row sum = 1 and tps in (0,1)
  foo <- log(gamma/diag(gamma))
  tgamma <- as.vector(foo[!diag(N)])
  # no values for delta needed in case if stationarity 
  if(stationary){
    tdelta <- NULL
  }
  # mlogit transformation for delta in case of non-stationarity
  else{
    tdelta<-log(delta[-1]/delta[1])
  }
  return(c(as.vector(tbeta),tgamma,tdelta))
}


### 3c) w2n_hmmissa (internal function) #####

# back-transformation from working (unconstraint) parameter vector to natural (constraint) parameters

# function input:
# * parvect: vector of transformed variables as produced by function n2w_hmmissa
# * N: number of states for the underlying latent MC
# * p:  number of all covariates including the ones belonging to sl and ta
# * sl_gamma: logical, TRUE if non-negative shape and rate parameters are to be estimated for gamma-distributed sl in movement kernel
# * ta_vonMises: logical, TRUE if non-negative kappa parameters are to be estimated for von Mises-distributed ta in movement kernel
# * correction: NULL if sl_gamma=ta_vonMises=FALSE, otherwise vector with correction values for shape, rate and kappa; vector c(1,0,0) for uniformly sampled available step lengths and angles, vector c(shape*,rate*,kappa*) for gamma-distributed step lengths with shape=shape* and rate=rate*, and von-Mises-distributed angles with concentration kappa*
# * stationary: logical, TRUE if stationarity for underlying latent MC is assumed
# function output:
# * list of back-transformed natural parameters
# * beta_clr: coefficient matrix containing the selection and movement coefficients for each covariate and state, respectively, dimension $p \times N$
# * gamma: tpm of MC, dimension $N\times N$
# * delta: initial distribution for MC, probability vector of dimension $1\times N$
# notes:
# * internal function

w2n_hmmissa<-function(parvect, N, p, sl_gamma=FALSE, ta_vonMises=FALSE, correction=NULL,
              stationary=FALSE){
  # re-create beta-matrix - but now including movement kernel parameters (beta_clr)
  beta_clr<-matrix(parvect[1:(N*p)],p,N)
  p_star<-p # index for kappa parameter - in case that sl_gamma=FALSE and ta_vonMises=TRUE
  # adjust coefficients for rate and shape if sl_gamma=TRUE
  # !note!: instead of shape and rate, shape-shape* and rate-rate* are used in clr (as in standard iSSA)
  # shape* and rate* depend on sampling procedure for sl of available steps
  # i) gamma-distr. sl: shape* and rate* are shape and rate values used for sampling
  # ii) uniform sampled sl: shape*=1 and rate*=0
  # !note!: shape and rate - related parameters are in last and second-last row
  if(sl_gamma){
    beta_clr[p-1,]<-exp(beta_clr[p-1,])-correction[1] # shape-shape*
    beta_clr[p,]<-exp(beta_clr[p,])-correction[2] # rate-rate*
    p_star<-p_star-2 # index for kappa if sl_gamma=TRUE
  }
  # adjust coefficient for kappa if ta_vonMises=TRUE
  # !note!: kappa-kappa* is used in clr instead of kappa (as in standard iSSA)
  # kappa* depend on sampling procedure for angle of available steps
  # i) von-Mises distr. ta: kappa* is kappa value used for sampling
  # ii) uniform sampled ta: kappa*=0
  # !note!: kappa - related parameters are in third-last row
  if(ta_vonMises){
    beta_clr[p_star,]<-exp(beta_clr[p_star,])-correction[3] # kappa-kappa*
  }
  # re-create tpm gamma
  gamma<-diag(N)
  gamma[!gamma]<-exp(parvect[N*p +1:(N*(N-1))])
  gamma<-gamma/apply(gamma,1,sum)
  # compute initial distr. delta
  if(stationary){
    delta<-solve(t(diag(N)-gamma+1),rep (1,N))
  }else{
    foo<-c(1,exp(parvect[(N*p+N*(N-1))+1:(N-1)]))
    delta<-foo/sum(foo)
  }
  return(list(beta_clr=beta_clr,gamma=gamma,delta=delta))
}


##### 3d) nlogLike_hmmissa (internal function) #####

# negative log-likelihood function

# function input:
# * parvect: vector of transformed variables as produced by n2w
# * X.list: double list containing design matrices X for each step and each ID, respectively; first list index belongs to ID, second list index belongs to stepID
# * N: number of states for the underlying latent MC
# * p:  number of all covariates including the ones belonging to sl and ta
# * nSteps: vector including the number of observed steps for each burst, respectively
# * vecIDs: vector including unique IDs in the data-frame
# * nID: number of unique IDs in data set
# * sl_gamma: logical, TRUE if non-negative shape and rate parameters are to be estimated for gamma-distributed sl in movement kernel
# * ta_vonMises: logical, TRUE if non-negative kappa parameters are to be estimated for von Mises-distributed ta in movement kernel
# * correction: NULL if sl_gamma=ta_vonMises=FALSE, otherwise vector with correction values for shape, rate and kappa; vector c(1,0,0) for uniformly sampled available step length and angles, vector c(shape*,rate*,kappa*) for gamma-distributed step length with shape=shape* and rate=rate*, and von-Mises-distributed angles with concentration kappa*
# * stationary: logical, TRUE if stationarity for underlying latent MC is assumed
# function output:
# * negative log-likelihood function value evaluated at parameter vector parvect
# notes:
# * internal function

nlogLike_hmmissa<-function(parvect, X.list, N ,p, nSteps, vecIDs, nID, sl_gamma, 
                   ta_vonMises,correction, stationary){
  # get natural parameters
  lpn<-w2n_hmmissa(parvect, N, p, sl_gamma, ta_vonMises, correction, stationary)
  # initialise log-likelihood value
  lscale_all<-0
  # loop across bursts
  for(run_ID in 1:nID){
    # compute matrix including clr-probs for each state and each step, respectively
    # matrix dimension TxN (T=nSteps[run_ID])
    # missing values are replaced by 1
    allprobs = t(sapply(X.list[[run_ID]],cpr_Nt,lpn$beta_clr))
    allprobs[is.na(allprobs) & !is.nan(allprobs)]<-1
    # forward algorithm to evaluate log-likelihood for running burst
    foo = lpn$delta * allprobs[1,] # t=1
    sumfoo = sum(foo)
    lscale = log(sumfoo) # current loglikelihood-value
    foo = foo / sumfoo
    for(ts in 2:nSteps[run_ID]) { # loop across all time-steps
      foo = foo %*% lpn$gamma * allprobs[ts,]
      sumfoo = sum(foo)
      lscale = lscale + log(sumfoo) # updated loglikelihood-value at t=ts
      foo = foo / sumfoo
    }
    lscale_all<-lscale_all+lscale # sum log-likelihood values of all bursts
  }
  # return negative log-likelihood value (as nlm minimises functions)
  return(-lscale_all)
}


##### 3e) mle_hmmissa #####

# maximum-likelihood estimation for HMM-(i)SSF

# function input:
# * formula: formula for clr of the form "used ~ cov_1 + ... + cov_p* + cos(ta-mean_vonMises) + log(sl) + (-sl) + strata(stepID)"; 
#           cos(ta-mean_vonMises) is used for von-Mises distributed turning angles * if ta_vonMises=TRUE, it must be placed behind the $p^{*}$ "selection" covariates; 
#           log(sl) and (-sl) used for gamma distributed step length * if sl_gamma=True, they must be placed at the end of formula in the given order
# * data: case-control data set; must include a column indicating case or control, a column for ID (for bursts/individuals), a column with stepID (strata variable); 
#           all rows belonging to one stepID (used and available) must appear one below the other with observed step coming first
# * N: number of states for the underlying latent MC
# * beta: initial values for matrix of selection coefficients, no constraints applied; dimension $p\times N$ for sl_gamma=ta_vonMises=FALSE or $p* \times N$ for sl_gamma=TRUE and/or ta_vonMises=FALSE
# * gamma: initial values for tpm of underlying latent MC, all elements lie in (0,1) and rows sum to 1; dimension $N\times N$
# * delta: initial values for initial distribution for MC, NULL if stationarity is assumed, otherwise probability vector with elements in (0,1) which sum to 1; dimension $1\times N$
# * shape: initial values for shape parameter vector for gamma dist. in constraint (sl_gamma=TRUE) estimation, NULL otherwise; dimension $1\times N$
# * rate: initial values for rate parameter vector for gamma dist. in constraint (sl_gamma=TRUE) estimation, NULL otherwise; dimension $1\times N$
# * kappa: initial values for kappa parameter vector for von Mises dist. in constraint (ta_vonMises=TRUE) estimation, NULL otherwise; dimension $1\times N$
# * sl_gamma: logical, TRUE if non-negative shape and rate parameters are to be estimated for gamma-distributed sl in movement kernel
# * ta_vonMises: logical, TRUE if non-negative kappa parameters are to be estimated for von Mises-distributed ta in movement kernel
# * correction: NULL if sl_gamma=ta_vonMises=FALSE, otherwise vector with correction values for shape, rate and kappa; vector c(1,0,0) for uniformly sampled available step lengths and angles, vector c(shape*,rate*,kappa*) for gamma-distributed step lengths with shape=shape* and rate=rate*, and von-Mises-distributed angles with concentration kappa*
# * ID: name of ID-column, character
# * ordering: ordering of the states, possible options are "mean_sl" (order states by step length mean values, lowest first, requires sl_gamma=TRUE),
#             "gamma" (order states by tpm diagonal, highest values first), "beta" (order states by selection coefficients for first covariate, lowest first)
#              and "n" (no specific order)
# * stationary: logical, TRUE if stationarity for underlying latent MC is assumed
# * iterlim: maximum number of iterations for nlm, default is 1000
# * gradtol: tolerance value for convergence in nlm, default is 1e-06 (default of nlm)
# * print.level: .print level for nlm, default is 2(all iterations printed)
# * stepmax: maximum allowable scaled step length in nlm, default NULL corresponds to nlm default
# * hessian: logical, if TRUE, nlm returns hessian; default is FALSE
# function output:
# * List of
# * beta: matrix of estimated selection coefficients for each covariate and state, respectively; dimension $p^{*} \times N$
# * shape: $N$-dimensional vector of estimated shape parameters if sl_gamma=TRUE, NULL otherwise
# * rate: $N$-dimensional vector of estimated rate parameters if sl_gamma=TRUE, NULL otherwise
# * kappa: $N$-dimensional vector of estimated kappa parameters if ta_vonMises=TRUE, NULL otherwise
# * gamma: estimated tpm; dimension $N \times N$
# * delta: initial distribution, i.e. $N$-dimensional probability vector as either estimated for stationarity=FALSE or as implied by the estimated tpm gamma if stationarity=TRUE
# * beta_clr: matrix of estimated clr coefficients for each covariate and state, respectively; dimension $p \times N$ (equal to beta in case of sl_gamma=ta_vonMises=FALSE)
# * nllk: minimum negative log-likelihood value found by nlm
# * estimate: unconstrained estimate vector
# * nit: number of iterations used by nlm
# * conv: convergence code of nlm
# * hessian: hessian output of nlm if hessian=TRUE, NULL otherwise
# notes:
# * estimation not possible with this function for $N=1$ states, i.e. for iSSAs without state-switching

mle_hmmissa<-function(formula, data, N, beta, gamma, delta,
                    shape, rate, kappa,
                    sl_gamma, ta_vonMises, correction, ID, ordering = "mean_sl",
                    stationary = FALSE, iterlim = 1000, gradtol = 1e-06, 
                    print.level = 2, stepmax=NULL, hessian=FALSE){
  # 1) step IDs and covariates
  # place of stepID in formula
  stepID<-as.numeric(attr(terms(formula, "strata"), "specials"))-1
  # vector of covariate names in formula
  covs<-all.vars(formula[[3]])[-stepID]
  # number of covariates
  p<-length(covs)
  # name of stepID variable
  stepID<-all.vars(formula[[3]])[stepID]
  # name of used column
  used<-all.vars(formula[[2]])
  # vector of data set columns used as covariates
  covs<-match(covs,colnames(data))
  # 2) bursts
  # column including IDs
  if(is.null(ID)){
    data$ID <- 1
    ID = "ID"
  }
  ID<-which(colnames(data)==ID)
  # vector including unique IDs
  vecIDs<-c(unique(data[,ID]))
  # number of IDs
  nID<-length(vecIDs)
  # vector for number of steps per ID
  nSteps<-numeric(nID)
  # X.list: double list including design matrix X for each step and each ID
  # first list index: ID
  # second list index: step
  X.list<-list()
  for(run_ID in 1:nID){ # loop across IDs
    # data for running ID
    ind_k<-which(data[,ID]==vecIDs[run_ID])
    data.k<-data[ind_k,]
    # number of steps for running ID
    nSteps[run_ID]<-sum(data.k[used])
    # row indicating beginning of new step
    aID <- which(!duplicated(data.k[stepID]))
    # nrow(data.k)+1 necessary to include to derive last row for each step ID
    aID <- c(aID,nrow(data.k)+1)
    X.list[[run_ID]]<-list()
    for(ts in 1:nSteps[run_ID]){ # loop across step IDs
      X.list[[run_ID]][[ts]]<-as.matrix(data.k[aID[ts]:(aID[ts+1]-1),covs])
    }
  }
  # compute working parameters for initial values
  parvect<-n2w_hmmissa(N, beta, gamma, delta, shape, rate, kappa, 
               stationary)
  # set stepmax value for nlm
  if(is.null(stepmax)){
    stepmax = max(1000 * sqrt(sum((parvect)^2)), 1000)
  }
  # use nlm to find maximum likelihood estimates
  mod<-nlm(nlogLike_hmmissa,parvect, X.list, N ,p, nSteps, vecIDs, nID, sl_gamma, 
           ta_vonMises,correction, stationary,
           iterlim=iterlim,gradtol=gradtol,print.level=print.level,
           stepmax=stepmax,hessian=hessian)
  # calculate natural parameters from maximum likelihood estimate vector
  pn <- w2n_hmmissa(mod$estimate, N, p, sl_gamma, ta_vonMises, correction,stationary)
  # add hessian object if hessian not computed
  if(is.null(hessian)){
    mod$hessian=NULL
  }
  # number of selection covariates
  p_star<-p
  # add shape, rate and kappa parameters
  shape<-rate<-kappa<-NULL
  # compute shape parameters if sl_gamma=TRUE
  if(sl_gamma){
    shape<-pn$beta_clr[p-1,]+correction[1]
    rate<-pn$beta_clr[p,]+correction[2]
    p_star<-p_star-2
  }
  # compute kappa parameters if ta_vonMises=TRUE
  if(ta_vonMises){
    kappa<-pn$beta_clr[p_star,]+correction[3]
    p_star<-p_star-1
  }
  # state order
  if(ordering=="n"){
    idx_ordering <- 1:N
  }else if(ordering=="mean_sl"){
    idx_ordering <- order(shape/rate)
  }else if(ordering=="gamma"){
    idx_ordering <- order(diag(pn$gamma),decreasing=TRUE)
  }else if(ordering=="beta"){
    idx_ordering<-order(pn$beta[1,])
  }
  pn$beta_clr<-pn$beta_clr[,idx_ordering]
  pn$gamma<-pn$gamma[idx_ordering,idx_ordering]
  pn$delta<-pn$delta[idx_ordering]
  if(sl_gamma){
    shape<-shape[idx_ordering]
    rate<-rate[idx_ordering]
  }
  # compute kappa parameters if ta_vonMises=TRUE
  if(ta_vonMises){
    kappa<-kappa[idx_ordering]
  }
  # beta without constraint movement parameters
  if(p_star>0){
    beta<-matrix(pn$beta_clr[1:p_star,],p_star,N)
  }else{
    beta<-NULL
  }
  return(list(beta=beta, shape=shape,
              rate=rate, kappa=kappa,
              gamma=pn$gamma,
              delta=pn$delta, beta_clr=pn$beta_clr,
              nllk=mod$minimum,estimate=mod$estimate,nit=mod$iterations,
              conv=mod$code,hessian=mod$hessian))
}


##### 3f) hessian_hmmissa #####

# hessian, standard errors, p-values and CIs for HMM-(i)SSF

# function input:
# * mod_hmmissa: HMM-(i)SSA model object as provided by mle_hmmissa-function
# * formula: formula as used to fit mod_hmmissa
# * data: data as used to fit mod_hmmissa
# * correction: correction values as used to fit mod_hmmissa
# * ID: column name in data indicating burst/animal IDs
# * stationary: logical if stationarity is assumed, value as used for fitting mod_hmmissa
# * alpha: alpha level for confidence intervals, default=0.05
# function output:
# * List of
# * hessian: Hessian matrix for HMM-(i)SSA model mod_hmmissa
# * se_beta: matrix of standard errors for selection coefficients in mod_hmmissa$\$$beta; dimension $p^{*}\times N$
# * CI_up_beta: matrix of upper confidence interval limits for selection coefficients in mod_hmmissa$\$$beta; dimension $p^{*}\times N$
# * CI_low_beta: matrix of lower confidence interval limits for selection coefficients in mod_hmmissa$\$$beta; dimension $p^{*}\times N$
# * p_beta: matrix of p-values (Wald test) for selection coefficients in mod_hmmissa$\$$beta; dimension $p^{*}\times N$
# notes:
# * could be integrated in mle_hmmissa function

hessian_hmmissa<-function(mod_hmmissa, formula, data, correction, stationary, ID, 
                        alpha=0.05){
  # 0) model features:
  # number of states
  N<-ncol(mod_hmmissa$beta_clr)
  sl_gamma<-(!is.null(mod_hmmissa$shape))
  ta_vonMises<-(!is.null(mod_hmmissa$kappa))
  # 1) step IDs and covariates
  # place of stepID in formula
  stepID<-as.numeric(attr(terms(formula, "strata"), "specials"))-1
  # vector of covariate names in formula
  covs<-all.vars(formula[[3]])[-stepID]
  # number of covariates
  p<-length(covs)
  # name of stepID variable
  stepID<-all.vars(formula[[3]])[stepID]
  # name of used column
  used<-all.vars(formula[[2]])
  # vector of data set columns used as covariates
  covs<-match(covs,colnames(data))
  # 2) bursts
  # column including IDs
  if(is.null(ID)){
    data$ID <- 1
    ID = "ID"
  }
  ID<-which(colnames(data)==ID)
  # vector including unique IDs
  vecIDs<-c(unique(data[,ID]))
  # number of IDs
  nID<-length(vecIDs)
  # vector for number of steps per ID
  nSteps<-numeric(nID)
  # X.list: double list including design matrix X for each step and each ID
  # first list index: ID
  # second list index: step
  X.list<-list()
  for(run_ID in 1:nID){ # loop across IDs
    # data for running ID
    ind_k<-which(data[,ID]==vecIDs[run_ID])
    data.k<-data[ind_k,]
    # number of steps for running ID
    nSteps[run_ID]<-sum(data.k[used])
    # row indicating beginning of new step
    aID <- which(!duplicated(data.k[stepID]))
    # nrow(data.k)+1 necessary to include to derive last row for each step ID
    aID <- c(aID,nrow(data.k)+1)
    X.list[[run_ID]]<-list()
    for(ts in 1:nSteps[run_ID]){ # loop across step IDs
      X.list[[run_ID]][[ts]]<-as.matrix(data.k[aID[ts]:(aID[ts+1]-1),covs])
    }
  }
  # compute working parameters for initial values
  parvect<-n2w_hmmissa(N, mod_hmmissa$beta, mod_hmmissa$gamma, mod_hmmissa$delta, 
               mod_hmmissa$shape, mod_hmmissa$rate, mod_hmmissa$kappa, 
               stationary)
  # compute hessian
  hes<-hessian(nlogLike_hmmissa, parvect, X.list=X.list, N=N, p=p, nSteps=nSteps, 
               vecIDs=vecIDs, nID=nID, sl_gamma=sl_gamma, ta_vonMises=ta_vonMises,
               correction=correction, stationary=stationary)
  # standard errors for elements in beta
  # number of selection coefficients for selection function (not movement kernel)
  p_beta<-p
  if(sl_gamma){
    p_beta<-p_beta-2
  }
  if(ta_vonMises){
    p_beta<-p_beta-1
  }
  # standard errors for beta_clr
  try(std.err<-sqrt(diag(ginv(hes))[c(1:(N*p))]))
  if(exists("std.err") & p_beta>0){
    # standard errors for beta
    se_beta<-matrix(std.err,ncol=N)[1:p_beta,]
    # confidence limits
    CI_up<-mod_hmmissa$beta+qnorm(1-alpha/2)*se_beta
    CI_low<-mod_hmmissa$beta+qnorm(alpha/2)*se_beta
    p_beta<-abs(mod_hmmissa$beta/se_beta)
    p_beta<-(1-pnorm(p_beta))*2
  }else{
    p_beta<-max(1,p_beta)
    se_beta<-CI_up<-CI_low<-p_beta<-matrix(NA,p_beta,N)
  }
  return(list(hessian=hes,se_beta=se_beta,CI_up_beta=CI_up,CI_low_beta=CI_low,p_beta=p_beta))
}


##### 3g) viterbi_hmmissa #####

# Viterbi decoding for HMM-(i)SSA

# function input:
# * mod_hmmissa: HMM-(i)SSF model object as provided by mle_hmmissa
# * formula: formula as used to fit mod_hmmissa
# * data: case-control data with structure as described above
# * ID: column name in data indicating burst/animal IDs
# function output:
# * List of Viterbi-decoded state sequences for each ID, respectively

viterbi_hmmissa<-function(mod_hmmissa, formula, data , ID){
  # 0) number of states:
  N <- ncol(mod_hmmissa$beta)
  # 1) step IDs and covariates
  # place of stepID in formula
  stepID<-as.numeric(attr(terms(formula, "strata"), "specials"))-1
  # vector of covariate names in formula
  covs<-all.vars(formula[[3]])[-stepID]
  # number of covariates
  p<-length(covs)
  # name of stepID variable
  stepID<-all.vars(formula[[3]])[stepID]
  # name of used column
  used<-all.vars(formula[[2]])
  # vector of data set columns used as covariates
  covs<-match(covs,colnames(data))
  # 2) bursts
  # column including IDs
  if(is.null(ID)){
    data$ID <- 1
    ID = "ID"
  }
  ID<-which(colnames(data)==ID)
  # vector including unique IDs
  vecIDs<-c(unique(data[,ID]))
  # number of IDs
  nID<-length(vecIDs)
  # Viterbi algorithm for each burst separately - results saved in list
  Vit.list<-list()
  for(run_ID in 1:nID){ # loop across IDs
    # data for running ID
    ind_k<-which(data[,ID]==vecIDs[run_ID])
    data.k<-data[ind_k,]
    # number of steps for running ID
    nSteps_run<-sum(data.k[used])
    # row indicating beginning of new step
    aID <- which(!duplicated(data.k[stepID]))
    # nrow(data.k)+1 necessary to include to derive last row for each step ID
    aID <- c(aID,nrow(data.k)+1)
    # X.list: list including design matrix X for each step
    X.list<-list()
    for(ts in 1:nSteps_run){ # loop across step IDs
      X.list[[ts]]<-as.matrix(data.k[aID[ts]:(aID[ts+1]-1),covs])
    }
    # allprobs matrix as in nLogLike
    allprobs = t(sapply(X.list,cpr_Nt,mod_hmmissa$beta_clr))
    allprobs[is.na(allprobs) & !is.nan(allprobs)]<-1
    xi <- matrix(0,nSteps_run,N)
    foo <- mod_hmmissa$delta*allprobs[1,]
    xi[1,] <- foo/sum(foo)
    for (i in 2:nSteps_run){
      foo <- apply(xi[i-1,]*mod_hmmissa$gamma,2,max)*allprobs[i,]
      xi[i,] <- foo/sum(foo)
    }
    iv <- numeric(nSteps_run)
    iv[nSteps_run] <-which.max(xi[nSteps_run,])
    for (i in (nSteps_run-1):1){
      iv[i] <- which.max(mod_hmmissa$gamma[,iv[i+1]]*xi[i,])
    }
    Vit.list[[run_ID]]<-iv
  }
  return(Vit.list)
}


##### 4) HMM estimation #####

# code to fit HMM with state-dependent gamma distributions for step length and 
# state-dependent von Mises distributions with mean zero for turning angle

### Data requirements ###

# Data requires a certain structure:
# * must contain a column named sl for step length
# * must contain a column named ta for turning angle

### Code structure ###

# Parameters are estimated using the function "mle_HMM". It embeds the maximisation of the log-likelihood-function
# (i.e., the minimisation of the negative log-likelihood function nlogLike_HMM) using nlm. To avoid parameters outside of their parameter space 
# (e.g. probabilities greater 1 or lower zero in the transition probability matrix, short tpm), 
# the parameters are transformed before the maximisation using the function n2w_HMM. 
# After the maximisation, parameters are back-transformed to their natural form using the function w2n_HMM. 

# functions for HMM estimation:
# a) internal functions without ta: n2w_HMM, w2n_HMM, nlogLike_HMM
# b) mle_HMM: function to estimate the HMM parameters using numerical optimisation (nlm),
#    uses n2w_HMM, w2n_HMM and nlogLike_HMM
# c) viterbi_HMM: internal function; negative log-likelihood function


##### 4a) internal functions  #####

# function that performs transformation
n2w_HMM <- function(shape,scale,kappa,gamma,N){
  tshape <- log(shape)
  tscale <- log(scale)
  tkappa <- log(kappa)
  foo <- log(gamma/diag(gamma))
  tgamma <- as.vector(foo[!diag(N)])
  parvect <- c(tshape,tscale,tkappa,tgamma)
  return(parvect)
}

# function that performs the inverse transformation
w2n_HMM <- function(parvect,N){
  shape <- exp(parvect[1:N])
  scale <- exp(parvect[(N+1):(2*N)])
  kappa <- exp(parvect[(2*N+1):(3*N)])  
  gamma <- diag(N)
  gamma[!gamma] <- exp(parvect[(3*N)+1:(N*(N-1))])
  gamma <- gamma/apply(gamma,1,sum)
  delta <- solve(t(diag(N)-gamma+1),rep(1,N))
  return(list(shape=shape,scale=scale,kappa=kappa,gamma=gamma,delta=delta))
}

# function that computes negative log-likelihood value of the movement HMM
nlogLike_HMM <- function(parvect,data,N){
  lpn <- w2n_HMM(parvect,N)
  n <- nrow(data)
  allprobs <- matrix(1,n,N)
  ind_sl<-which(!is.na(data$sl))
  ind_ta<-which(!is.na(data$ta))
  for (j in 1:N){
    allprobs[ind_sl,j] <- dgamma(data$sl[ind_sl],
                                   shape=lpn$shape[j],scale=lpn$scale[j])
    allprobs[ind_ta,j] <- allprobs[ind_ta,j]*dvm(data$ta[ind_ta],
                                                       mu=0,kappa=lpn$kappa[j])
  }
  foo <- lpn$delta*allprobs[1,]
  sumfoo<-sum(foo)
  lscale <- log(sumfoo)
  foo<-foo/sumfoo
  for (i in 2:n){
    foo <- foo%*%lpn$gamma*allprobs[i,]  
    sumfoo <- sum(foo); lscale <- lscale+log(sumfoo); foo <- foo/sumfoo # scaling
  }
  return(-lscale)
}


##### 4c) mle_HMM  #####

# maximum-likelihood estimation for movement HMM

# function input:
# * data: data frame with columns sl and ta
# * shape0: initial shape values for sl gamma distribution
# * scale0: initial scale values for sl gamma distribution
# * kappa0: initial concentration values for ta von Mises distribution
# * gamma0: initial transition probability matrix
# * stepmax: stepmax for nlm optimisation routine
# * ordering: ordering of the states, possible options are "mean_sl" (order states by step length mean values, lowest first),
#             "gamma" (order states by tpm diagonal, highest values first), "n" (no specific order)
# function output:
# * shape: $N$-dimensional vector of estimated shape parameters
# * rate: $N$-dimensional vector of estimated rate parameters
# * kappa: $N$-dimensional vector of estimated concentration parameters
# * gamma: estimated tpm; dimension $N \times N$
# * delta: stationary distribution as implied by the estimated tpm gamma
# * nllk: minimum negative log-likelihood value found by nlm
# * estimate: unconstrained and unordered estimate vector
# * conv: convergence code of nlm
# * nit: number of iterations used by nlm
# * gradient: gradient vector of vector estimate

# function output:
mle_HMM <- function(data,shape0,scale0,kappa0,gamma0,stepmax=150,ordering="mean_sl"){
  # number of states
  N<-length(shape0)
  parvect <- n2w_HMM(shape0,scale0,kappa0,gamma0,N)
  mod <- nlm(nlogLike_HMM,parvect,data,N,print.level=1,iterlim=1000,stepmax=stepmax)
  pn <- w2n_HMM(mod$estimate,N)
  # state order
  if(ordering=="gamma"){
    idx_ordering <- order(diag(pn$gamma),decreasing=TRUE)
  }else if(ordering=="mean_sl"){
    idx_ordering <- order(pn$shape*pn$scale)
  }else if(ordering=="n"){
    idx_ordering<-1:N
  }
  return(list(shape=pn$shape[idx_ordering],scale=pn$scale[idx_ordering],
              kappa=pn$kappa[idx_ordering],
              gamma=pn$gamma[idx_ordering,idx_ordering],delta=pn$delta[idx_ordering],
              nllk=mod$minimum,conv=mod$code,nit=mod$iterations,
              gradient=mod$gradient))
}


##### 4c) viterbi_HMM  #####

# Viterbi decoding for movement HMM

# function input:
# * data: data frame with columns sl and ta
# * mod: HMM model object as provided by mle_HMM
# * N: number of states
# function output:
# * Viterbi-decoded state sequence

viterbi_HMM<-function(data,mod,N){
  n<-nrow(data)
  allprobs <- matrix(1,n,N)
  ind_sl<-which(!is.na(data$sl))
  ind_ta<-which(!is.na(data$ta))
  for (j in 1:N){
    allprobs[ind_sl,j]<-dgamma(data$sl[ind_sl],shape=mod$shape[j],scale=mod$scale[j])
    allprobs[ind_ta,j]<-allprobs[ind_ta,j]*dvm(data$ta[ind_ta],mu=0,kappa=mod$kappa[j])
  }
  # loop
  xi <- matrix(0,n,N)
  foo <- mod$delta*allprobs[1,]
  xi[1,] <- foo/sum(foo)
  for (i in 2:n){
    foo <- apply(xi[i-1,]*mod$gamma,2,max)*allprobs[i,]
    xi[i,] <- foo/sum(foo)
  }
  iv <- numeric(n)
  iv[n] <-which.max(xi[n,])
  for (i in (n-1):1){
    iv[i] <- which.max(mod$gamma[,iv[i+1]]*xi[i,])
  }
  iv
}


##### 5) wrapper-functions for parallel computation #####

##### 5a) data simulation #####

# function input:
# * i_run: index for simulation run
# * remaining arguments: as for simdata_hmmissa, except that seed is a vector of seed values
# function output:
# * list of data sets as returned by simdata_hmmissa

simdata_hmmissa_par <- function(i_run,parameter, Ts, landscape, seed, n_control, center, ext_hr){
  simdata_hmmissa(parameter, Ts, landscape, seed=seed[i_run], n_control, center, ext_hr=ext_hr)
}

##### 5b) fit gamma distribution to step length #####

# function input:
# * data: list of data sets of same dimension as provided by simdata_hmmissa_par
# * Ts: integer, number of rows for data frames in data
# function output:
# * matrix of rate and shape parameters for each data frame in data

fit.gamma_par <- function(data,Ts){
  param_g<-fitdist(data$sl[-c(1,Ts)],
                   distr = "gamma", method = "mle")
  param_g<-param_g$estimate
  return(param_g)
}

##### 5c) HMM Viterbi decoding #####

# function input:
# * i_run: index for data set list
# * data: data list
# * mod: HMM model object as provided by mle_HMM
# * N: number of states
# function output
# * list of Viterbi-decoded state sequences

viterbi_HMM_par<-function(i_run,data,mod,N){
  vit.h<-viterbi_HMM(data[[i_run]],mod=mod[[i_run]],N=N)
  return(vit.h)
}


##### 5d) case control data set #####

# function input:
# * i_run: index for data set list
# * remaining arguments as used for dat_case_cont, except that data is a list of data sets; 
#   seed is a vector of seed values and para_sampling is a matrix of parameters for control sampling
# function output
# * list of case-control data sets

dat_case_cont_par <- function(i_run,data,landscape,n_control,Ts,seed, dist_sl, para_sampling){
  dat.h <- dat_case_cont(data[[i_run]],landscape,n_control,Ts,seed[i_run], 
                         dist_sl, para_sampling[i_run,])
  return(dat.h)
}

# function input:
# * i_run: index for data set list
# * remaining arguments as used for dat_TS_case_cont, except that data is a list of data sets; 
#   s.vit is a list of Viterbi-decoded state sequences, seed is a vector of seed values 
#   and mod_HMM is a list of HMM model objects
# function output
# * list of case-control data sets

dat_TS_case_cont_par <- function(i_run,data,s.vit,landscape,n_control,Ts,seed, mod_HMM){
  dat.h <- dat_TS_case_cont(data[[i_run]],s.vit[[i_run]],landscape,n_control,Ts,
                            seed[i_run], mod_HMM[[i_run]])
  return(dat.h)
}


##### 5d) iSSA #####

# function input:
# * data: list of case-control data sets as provided by dat_case_cont_par
# * formula: iSSA formula
# function output
# * list of iSSA objects consisting of clogit-summaries (survival package)

issa_par<-function(data,formula){
  mod_iSSA<-clogit(formula,data=data)
  return(summary(mod_iSSA))
}

##### 5e) HMM-iSSA #####

# function input:
# * i_run: index for data set list
# * remaining arguments as used for mle_hmmissa and hessian_hmmissa except that 
#   data is a list of data sets, correction is a matrix of correction values
# function output
# * list of HMM-iSSA objects as provided by mle_hmmissa and additionally hessian_hmmissa

hmmissa_par<-function(i_run, formula, data, N, beta0, gamma0, delta0,
                         shape0, rate0, kappa0, sl_gamma, ta_vonMises,
                         correction, ordering, stationary, stepmax){
  try({
    mod<-mle_hmmissa(formula, data[[i_run]], N=N, beta=beta0, gamma=gamma0, delta=delta0,
                    shape=shape0, rate=rate0, kappa=kappa0,
                    sl_gamma=sl_gamma, ta_vonMises=ta_vonMises,
                    correction=correction[i_run,],
                    ID=NULL, ordering=ordering,
                    stationary = stationary, print.level = 0,
                    stepmax=stepmax)
    hessian_nlm<-hessian_hmmissa(mod,formula, data[[i_run]],
                             correction=correction[i_run,],
                             stationary = stationary, ID=NULL, alpha=0.05)
    mod$hessian<-hessian_nlm$hes
    mod$se_beta<-hessian_nlm$se_beta
    mod$CI_up_beta<-hessian_nlm$CI_up_beta
    mod$CI_low_beta<-hessian_nlm$CI_low_beta
    mod$p_beta<-hessian_nlm$p_beta
  },silent = TRUE)
  if(!exists('mod')){
    mod<-NULL
  }
  if(exists('mod') & !exists('hessian_nlm')){
    mod$hessian<-NA
    mod$se_beta<-NA
    mod$CI_up_beta<-NA
    mod$CI_low_beta<-NA
    mod$p_beta<-NA
  }
  return(mod)
}


##### 5f) TS-iSSA #####

# function to estimate TS-iSSA coefficients

# function input:
# * data: list of case-control data sets as provided by dat_case_cont_par
# * formula: TSiSSA formula
# * state: state to estimate TSiSSA for
# * nstate_thresh: minimum number of observations required to estimate TS-iSSA estimates
# function output
# * list of TSiSSA objects consisting of clogit-summaries (survival package)

tsissa_par<-function(data,formula,state,nstate_thresh=0){
  ind_state<-which(data$state==state)
  mod_TSiSSA<-NA
  if(length(ind_state)>nstate_thresh){
    try(mod.h<-clogit(formula,data=data[ind_state,]),silent = TRUE)
    if(exists("mod.h")){
      mod_TSiSSA<-summary(mod.h)
    }
  }
  return(mod_TSiSSA)
}


##### 5g) HMM-iSSA Viterbi decoding #####

# Viterbi decoding for HMM-iSSA

# function input:
# * i_run: index for simulation run
# * remaining arguments as used for vit_hmmissa and dat_case_cont except that
#   mod_hmmissa is a list of HMM-iSSA model objects, data is a list of movement data sets
#   and para_sampling is a list of parameter values required for step length control sampling
# function otput
# * list of Viterbi-decoded HMM-iSSA state sequences

vit_hmmissa_par<-function(i_run,mod_hmmissa,data,landscape,formula,Ts,
                         n_control,dist_sl,para_sampling,seed,ID=NULL){
  if(is.null(mod_hmmissa[[i_run]]) | length(mod_hmmissa[[i_run]])<11){
    s.vit<-rep(NA,Ts-2)
  }else{
    # create case-control data
    data_cc <-dat_case_cont(data=data[[i_run]], landscape=landscape,
                            n_control=n_control,Ts=Ts,seed=seed[i_run],
                            dist_sl=dist_sl,para_sampling=para_sampling[i_run,])
    s.vit<-viterbi_hmmissa(mod_hmmissa=mod_hmmissa[[i_run]], formula=formula,
                         data=data_cc, ID=ID)
  }
  return(s.vit)
}


##### 5h) HMM-iSSA for different starting values (case study) #####

# iv_run: index for initial values set
# * remaining arguments as used for mle_hmmissa and hessian_hmmissa except that
#   shape, rate and d.h are matrices including different starting value sets for shape, rate and
#   the tpm diagonal and beta is a list of initial selection coefficient matrices
# function output:
# * list of HMM-iSSA model objects

hmmissa_iv_par<-function(iv_run,formula, data, N, beta, shape, rate, d.h, delta, 
                    correction,ID,stepmax=NULL){
  gamma0<-matrix(c(d.h[iv_run,1],1-d.h[iv_run,1],1-d.h[iv_run,2],d.h[iv_run,2]),
                 N,N,byrow=TRUE)
  if(!is.null(beta)){
    beta_run<-beta[[iv_run]]
  }else{
    beta_run<-NULL
  }
  try(mod<-mle_hmmissa(formula=formula, data=data, N=N, beta=beta_run, 
                      shape=shape[iv_run,], rate=rate[iv_run,], kappa=NULL,
                      gamma=gamma0, delta=delta,
                      sl_gamma=TRUE, ta_vonMises=FALSE,
                      correction=correction,
                      ID=ID, ordering = "mean_sl",
                      stationary = FALSE, iterlim = 1000, print.level = 0, 
                      stepmax=stepmax, hessian=FALSE))
  if(exists("mod")){
    mod$se_beta<-NA
    mod$CI_low_beta<-NA
    mod$CI_up_beta<-NA
    mod$p_beta<-NA
    try(hessian<-hessian_hmmissa(mod_hmmissa=mod,formula=formula, data=data, 
                                correction=correction, stationary=FALSE,ID=ID, 
                                alpha=0.05))
    if(exists("hessian")){
      mod$se_beta<-hessian$se_beta
      mod$CI_low_beta<-hessian$CI_low_beta
      mod$CI_up_beta<-hessian$CI_up_beta
      mod$p_beta<-hessian$p_beta
    }
  }else{
    mod<-list()
    mod$nllk<-NA
  }
  return(mod)
}

